package no.ntnu.idatt1002.demo.controller;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ListView;
import javafx.scene.input.MouseEvent;
import javafx.stage.Stage;
import no.ntnu.idatt1002.demo.data.recipes.*;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;

public class AllRecipesController implements Initializable {

    IngredientsAtHand ingredientsAtHand;
    RecipeRegister recipeRegister;

    @FXML
    private Button goBackBtn;

    @FXML
    private ListView<String> allList;


    private ObservableList<String> recipes;

    private String selectedRecipeName;


    @FXML
    private void goBack(ActionEvent event) throws IOException {
        FXMLLoader loader = new FXMLLoader();
        loader.setLocation(getClass().getResource("/view/SuggestRecipes.fxml"));

        Parent root = loader.load();
        Stage stage = (Stage) ((Node) event.getSource()).getScene().getWindow();
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.show();
    }

    private void showRecipe(String recipeName) throws IOException {
        FXMLLoader loader = new FXMLLoader();
        loader.setLocation(getClass().getResource("/view/Recipe.fxml"));

        Recipe recipeOfInterest = recipeRegister.getRecipe(recipeName);

        Parent root = loader.load();

        RecipeController recipeController = loader.getController();

        recipeController.setData(recipeOfInterest);

        Stage stage = (Stage)allList.getParent().getScene().getWindow();
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.show();
    }

private float percent(int a, int b) {
        if(b != 0 && a != 0) {
            return (float) a / b;
        } else {
            return 0;
        }


}

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {

        ingredientsAtHand = FileHandler.readIngredientsAtHand("Fridge");
        recipeRegister = FileHandler.readRecipeRegister("Recipes");

        int numberOfRecipes = recipeRegister.getRecipes().size();

        ArrayList<Recipe> sortedRecipes = recipeRegister.pickBestFits(numberOfRecipes, ingredientsAtHand);

        recipes = FXCollections.observableArrayList(sortedRecipes.stream().map(recipe -> {
            return String.format("# %s  -  %d missing ingredients (%.2f %%)", recipe.getName(), recipe.getMissingIngredients(), percent(recipe.getIngredientList().size() - recipe.getMissingIngredients(),recipe.getIngredientList().size()));
        }).toList());

        allList.setItems(recipes);

        allList.setOnMouseClicked(new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent mouseEvent) {
                    selectedRecipeName = allList.getSelectionModel()
                            .getSelectedItem().split("-|#")[1].strip();

                try {
                   showRecipe(selectedRecipeName);

                } catch (IOException e) {
                    throw new RuntimeException(e);
                }

            }
        });

    }
}
