package no.ntnu.idatt1002.demo.controller;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import no.ntnu.idatt1002.demo.data.recipes.*;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Optional;
import java.util.ResourceBundle;

public class SuggestRecipesController implements Initializable {

    IngredientsAtHand ingredientsAtHand;
    RecipeRegister recipeRegister;


    @FXML
    private Button showAllBtn;

    @FXML
    private Button goBackBtn;

    @FXML
    private ListView<String> fridgeList;

    @FXML
    private GridPane recipeGrid;

    @FXML
    private Label missingList;

    private ObservableList<String> fridge;

    private final ArrayList<VBox> currentRecipeTiles = new ArrayList<>(4);



    @FXML
    private void addIngredient() throws IOException {
        FXMLLoader loader = new FXMLLoader();
        loader.setLocation(getClass().getResource("/view/AddIngredient.fxml"));
        DialogPane addIngredientPane = loader.load();

        Dialog<ButtonType> dialog = new Dialog<>();
        dialog.setDialogPane(addIngredientPane);
        dialog.setTitle("Add ingredient to fridge");

        Optional<ButtonType> clickedButton = dialog.showAndWait();

        if (clickedButton.isPresent() && clickedButton.get() == ButtonType.CLOSE) {
           readIngredientsAtHand();
            setRecipeTiles();
        }
    }

    @FXML
    private void removeFromFridge() throws IOException {
        String toRemove = fridgeList.getSelectionModel().getSelectedItem();
        if(toRemove != null) {
            ingredientsAtHand.removeIngredient(FoodItem.valueOf(toRemove.replace(" ", "_").toUpperCase()));
            storeIngredientsAtHand();
            setRecipeTiles();
        }
    }



    private void setRecipeTiles() {
        int NUMBER_OF_TILES = 4;
        ArrayList<Recipe> recipes = recipeRegister.pickBestFits(NUMBER_OF_TILES, ingredientsAtHand);

        int i = 0;
        int j = 0;
        int counter = 0;

        for (Recipe r : recipes) {
            FXMLLoader loader = new FXMLLoader();
            loader.setLocation(getClass().getResource("/view/RecipeTile.fxml"));

            if (i > 1) {
                j++;
                i = 0;
            }

            try {
                VBox vBox = loader.load();
                RecipeTileController recipeTileController = loader.getController();
                recipeTileController.setData(r);

                if (currentRecipeTiles.size() < recipes.size()) {
                    currentRecipeTiles.add(vBox);
                } else {
                    recipeGrid.getChildren().remove(currentRecipeTiles.get(counter));
                    currentRecipeTiles.set(counter, vBox);
                }

                setHoverEffect(vBox, r);

                recipeGrid.add(vBox, i, j);


            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            i++;
            counter++;
        }
    }


    private void setHoverEffect(VBox vBox, Recipe recipe) {
        vBox.setOnMouseEntered(event -> {
            if(recipe.getMissingIngredients()==0) {
                missingList.setText("");
                missingList.setVisible(false);
            } else {
                missingList.setText("Missing: " + String.join(", ", recipe.getMissingList()));
                missingList.setVisible(true);
            }

        });

        vBox.setOnMouseExited(event -> {
            missingList.setText("");
            missingList.setVisible(false);
        });
    }


    @FXML
    private void switchScene(ActionEvent event) throws IOException {
        FXMLLoader loader = new FXMLLoader();
        if (event.getSource() == showAllBtn) {
            loader.setLocation(getClass().getResource("/view/AllRecipes.fxml"));
        } else if (event.getSource() == goBackBtn) {
            loader.setLocation(getClass().getResource("/view/MainMenuNew.fxml"));
        }
        Parent root = loader.load();
        Stage stage = (Stage) ((Node) event.getSource()).getScene().getWindow();
        Scene scene = new Scene(root);
        stage.setScene(scene);
        stage.setResizable(false);
        stage.show();
    }

    public void readIngredientsAtHand() {
        ingredientsAtHand = FileHandler.readIngredientsAtHand("Fridge");
        if (ingredientsAtHand == null) {
            ingredientsAtHand = new IngredientsAtHand();
        }
        fridge = FXCollections.observableArrayList(ingredientsAtHand.getIngredientsAtHand().stream().map(foodItem -> foodItem.label).toList());
        fridgeList.setItems(fridge.sorted());
    }

    public void storeIngredientsAtHand() throws IOException {
        FileHandler.writeIngredientsAtHand(ingredientsAtHand, "Fridge");
        fridge = FXCollections.observableArrayList(ingredientsAtHand.getIngredientsAtHand().stream().map(foodItem -> foodItem.label).toList());
        fridgeList.setItems(fridge.sorted());
    }


    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {

        readIngredientsAtHand();
         recipeRegister = FileHandler.readRecipeRegister("Recipes");
         if(recipeRegister == null) {
             recipeRegister = new RecipeRegister();
         }
        //ObservableList<Recipe> recipes = FXCollections.observableArrayList(recipeRegister.getRecipes());
         missingList.setVisible(false);

         setRecipeTiles();
    }
}
